/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.HashMap;

public class CustomEditManager {

    private static Log log = LogFactory.getLog(CustomEditManager.class);

    private Map <String, EditProcessor> editProcessors = new HashMap <String, EditProcessor> ();

    public CustomEditManager() {

        // we should register built in edit processors here
        editProcessors.put(RegistryConstants.TEXT_EDIT_PROCESSOR_KEY, new TextEditProcessor());
    }

    public void addProcessor(String processorKey, EditProcessor editProcessor) {
        editProcessors.put(processorKey, editProcessor);
    }

    public void process(String path, HttpServletRequest request, HttpServletResponse response)
            throws RegistryException {

        String processorKey = request.getParameter(RegistryConstants.CUSTOM_EDIT_PROCESSOR_KEY);
        String editViewName = request.getParameter(RegistryConstants.EDIT_VIEW_NAME);
        if (processorKey == null) {
            String msg = "Processor key is not set in the request, " +
                    "which is generated by a custom edit UI.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        EditProcessor editProcessor = editProcessors.get(processorKey);
        if (editProcessor == null) {
            String msg = "Edit processor is not registered for processing " +
                    "the custom edit requests for key " + processorKey;
            log.error(msg);
            throw new RegistryException(msg);
        }

        editProcessor.processEdit(path, editViewName, request, response);
    }
}
