/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;

import java.util.Map;
import java.util.Iterator;

public abstract class UIEnabledHandler extends Handler {

    public Resource get(RequestContext requestContext) throws RegistryException {

        ResourcePath resourcePath = requestContext.getResourcePath();
        Resource resource;
        if (resourcePath.parameterExists(RegistryConstants.VIEW_PROPERTY)) {

            String viewName = resourcePath.getParameterValue(RegistryConstants.VIEW_PROPERTY);
            if (viewName == null || "".equals(viewName) ||
                    RegistryConstants.DEFAULT_VIEW_NAME.equals(viewName)) {
                viewName = getDefaultBrowseView();
            }

            if (RegistryConstants.RAW_VIEW_NAME.equals(viewName)) {
                resource = getRawResource(requestContext);

            } else {

                resource = getView(viewName, requestContext);

                if (resource.getProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY) == null) {

                    if (RegistryConstants.TEXT_VIEW_NAME.equals(viewName)) {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_VIEW_TEXT);
                    } else if (RegistryConstants.XML_VIEW_NAME.equals(viewName)) {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_VIEW_XML);
                    } else {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_GENERAL);
                    }
                }
            }

            resource.setProperty(
                    RegistryConstants.CONTENT_UI_MODE, RegistryConstants.BROWSE_MODE);

        } else if (resourcePath.parameterExists(RegistryConstants.EDIT_PROPERTY)) {

            String viewName = resourcePath.getParameterValue(RegistryConstants.EDIT_PROPERTY);
            if (viewName == null || "".equals(viewName) ||
                    RegistryConstants.DEFAULT_VIEW_NAME.equals(viewName)) {
                viewName = getDefaultEditView();
            }

            if (RegistryConstants.RAW_VIEW_NAME.equals(viewName)) {
                resource = getRawResource(requestContext);

            } else {

                resource = getEdit(viewName, requestContext);

                if (resource.getProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY) == null) {

                    if (RegistryConstants.TEXT_VIEW_NAME.equals(viewName)) {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_EDIT_TEXT);
                    } else if (RegistryConstants.XML_VIEW_NAME.equals(viewName)) {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_EDIT_XML);
                    } else {
                        resource.setProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY,
                                RegistryConstants.R_GENERAL);
                    }
                }
            }

            resource.setProperty(
                    RegistryConstants.CONTENT_UI_MODE, RegistryConstants.EDIT_MODE);

        } else {
            resource = getRawResource(requestContext);
        }

        fillViews(resource);
        return resource;
    }

    public Resource getRawResource(RequestContext requestContext)
            throws RegistryException {

        if (requestContext.getResource() != null) {
            return requestContext.getResource();
        }

        if (requestContext.getResourcePath().isCurrentVersion()) {
            return requestContext.getRepository().get(requestContext.getResourcePath().getPath());
        }

        return null;
    }

    public abstract String[] getBrowseViews();

    public abstract String[] getEditViews();

    public String getDefaultBrowseView() {
        return RegistryConstants.RAW_VIEW_NAME;
    }

    public String getDefaultEditView() {
        return RegistryConstants.RAW_VIEW_NAME;
    }

    public abstract Resource getView(String viewName, RequestContext requestContext)
            throws RegistryException;

    public abstract Resource getEdit(String editName, RequestContext requestContext)
            throws RegistryException;

    private void fillViews(Resource resource) {

        String[] browseViews = getBrowseViews();
        for (int i = 0; i < browseViews.length; i++) {
            String browseView = browseViews[i];
            resource.addProperty("registry.browse-views", browseView);
        }

        String[] editViews = getEditViews();
        for (int i = 0; i < editViews.length; i++) {
            String editView = editViews[i];
            resource.addProperty("registry.edit-views", editView);
        }
    }
}
