/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Map;

public class XSLTBasedUIEnabledHandler extends UIEnabledHandler {

    private static Log log = LogFactory.getLog(XSLTBasedUIEnabledHandler.class);

    protected String viewXSLT;
    protected Transformer viewTransformer;

    protected String editXSLT;
    protected Transformer editTransformer;

    private TransformerFactory transformerFactory;

    public XSLTBasedUIEnabledHandler() {
        transformerFactory = TransformerFactory.newInstance();
    }

    public void setViewXSLT(String viewXSLT) throws RegistryException {
        this.viewXSLT = viewXSLT;
        InputStream xsltStream =
                Thread.currentThread().getContextClassLoader().getResourceAsStream(viewXSLT);

        try {
            viewTransformer = transformerFactory.newTransformer(new StreamSource(xsltStream));

        } catch (TransformerConfigurationException e) {
            String msg = "Failed to create transformer for the view UI XSLT. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void setEditXSLT(String editXSLT) throws RegistryException {
        this.editXSLT = editXSLT;
        InputStream xsltStream =
                Thread.currentThread().getContextClassLoader().getResourceAsStream(editXSLT);

        try {
            editTransformer = transformerFactory.newTransformer(new StreamSource(xsltStream));

        } catch (TransformerConfigurationException e) {
            String msg = "Failed to create transformer for the edit UI XSLT. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public String[] getBrowseViews() {
        return new String[0];
    }

    public String[] getEditViews() {
        return new String[0];
    }

    public Resource getView(String viewName, RequestContext requestContext)
            throws RegistryException {

        if (viewTransformer == null) {
            return getRawResource(requestContext);
        }

        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        Resource resource =
                requestContext.getRegistry().get(requestContext.getResourcePath().getPath());

        try {
            viewTransformer.setParameter("resourcePath",
                                         requestContext.getResourcePath().toString());
            viewTransformer.transform(
                    new StreamSource(resource.getContentStream()), new StreamResult(byteOut));

        } catch (TransformerException e) {
            String msg = "Failed to generate the view UI for resource " +
                    requestContext.getResourcePath() + ". XSLT transformation failed for XSLT " +
                    viewXSLT + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        String content = byteOut.toString();
        resource.setContent(content);

        return resource;
    }

    public Resource getEdit(String editName, RequestContext requestContext)
            throws RegistryException {

        if (editTransformer == null) {
            return getRawResource(requestContext);
        }

        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        Resource resource =
                requestContext.getRegistry().get(requestContext.getResourcePath().getPath());

        try {
            editTransformer.setParameter("resourcePath", requestContext.getResourcePath());
            editTransformer.transform(
                    new StreamSource(resource.getContentStream()), new StreamResult(byteOut));

        } catch (TransformerException e) {
            String msg = "Failed to generate the edit UI for resource " +
                    requestContext.getResourcePath() + ". XSLT transformation failed for XSLT " +
                    viewXSLT + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        String content = byteOut.toString();
        resource.setContent(content);

        return resource;
    }
}
